{-|
Module      : Gargantext.Core.Worker
Description : Asynchronous worker logic
Copyright   : (c) CNRS, 2024
License     : AGPL + CECILL v3
Maintainer  : team@gargantext.org
Stability   : experimental
Portability : POSIX

-}


module Gargantext.Core.Worker where


import Async.Worker.Broker.Redis (RedisBroker)
import Async.Worker.Broker.Types (BrokerMessage, toA, getMessage)
import Async.Worker qualified as Worker
import Async.Worker.Types qualified as Worker
import Async.Worker.Types (HasWorkerBroker)
import Data.Text qualified as T
import Database.Redis qualified as Redis
import Gargantext.Core.Worker.Jobs
import Gargantext.Core.Worker.TOML (WorkerDefinition(..))
import Gargantext.Prelude

  

withRedisWorker :: (HasWorkerBroker RedisBroker Job)
                => Redis.ConnectInfo
                -> WorkerDefinition
                -> (Async () -> Worker.State RedisBroker Job -> IO ())
                -> IO ()
withRedisWorker connInfo (WorkerDefinition { .. }) cb = do
  broker <- initializeRedisBroker connInfo

  let state' = Worker.State { broker
                            , queueName = _wdQueue
                            , name = T.unpack _wdName
                            , performAction
                            , onMessageReceived = Nothing
                            , onJobFinish = Nothing
                            , onJobTimeout = Nothing
                            , onJobError = Nothing }

  withAsync (Worker.run state') (\a -> cb a state')


performAction :: (HasWorkerBroker b Job)
              => Worker.State b Job
              -> BrokerMessage b (Worker.Job Job)
              -> IO ()
performAction _state bm = do
  let job' = toA $ getMessage bm
  case Worker.job job' of
    Ping -> putStrLn ("ping" :: Text)
