{-|
Module      : Gargantext.Core.Worker.Jobs
Description : Worker job definitions
Copyright   : (c) CNRS, 2024
License     : AGPL + CECILL v3
Maintainer  : team@gargantext.org
Stability   : experimental
Portability : POSIX

-}


module Gargantext.Core.Worker.Jobs where


import Async.Worker.Broker.Redis (RedisBroker, BrokerInitParams(RedisBrokerInitParams))
import Async.Worker.Broker.Types (Broker, initBroker)
import Async.Worker qualified as Worker
import Async.Worker.Types qualified as Worker
import Async.Worker.Types (HasWorkerBroker)
import Control.Lens (view)
import Data.Aeson ((.:), (.=), object, withObject)
import Data.Aeson.Types (prependFailure, typeMismatch)
import Database.Redis qualified as Redis
import Gargantext.API.Admin.Types (HasSettings, settings, workerSettings)
import Gargantext.Core.Worker.TOML (findDefinitionByName, WorkerDefinition(..))
import Gargantext.Database.Prelude (Cmd')
import Gargantext.Prelude


data Job =
  Ping
  deriving (Show, Eq)
instance FromJSON Job where
  parseJSON = withObject "Job" $ \o -> do
    type_ <- o .: "type"
    case type_ of
      "Ping" -> return Ping
      s -> prependFailure "parsing job type failed, " (typeMismatch "type" s)
instance ToJSON Job where
  toJSON Ping = object [("type" .= ("Ping" :: Text))]


initializeRedisBroker :: (HasWorkerBroker RedisBroker Job)
                      => Redis.ConnectInfo
                      -> IO (Broker RedisBroker (Worker.Job Job))
initializeRedisBroker connInfo = do
  let initParams = RedisBrokerInitParams connInfo

  initBroker initParams


sendJob :: (HasWorkerBroker RedisBroker Job, HasSettings env)
        => Redis.ConnectInfo
        -> Text
        -> Job
        -> Cmd' env err ()
sendJob connInfo workerName job = do
  ws <- view $ settings . workerSettings
  let mWd = findDefinitionByName ws workerName
  case mWd of
    Nothing -> panicTrace $ "worker definition not found for " <> workerName
    Just wd -> liftBase $ do
      b <- initializeRedisBroker connInfo
      let queueName = _wdQueue wd
      _ <- Worker.sendJob' $ Worker.mkDefaultSendJob' b queueName job
      pure ()
