{-|
Module      : Gargantext.Core.Worker.TOML
Description : Worker TOML file config
Copyright   : (c) CNRS, 2024
License     : AGPL + CECILL v3
Maintainer  : team@gargantext.org
Stability   : experimental
Portability : POSIX

-}


module Gargantext.Core.Worker.TOML where

import Async.Worker.Broker.Types qualified as Broker
import Data.Text qualified as T
import Database.Redis qualified as Redis
import Gargantext.Prelude
import Toml


type WorkerName = Text


data WorkerSettings =
  WorkerSettings {
    _wsDefinitions  :: ![WorkerDefinition]
  } deriving (Show, Eq)


data WorkerDefinition =
  WorkerDefinition {
    _wdName   :: !WorkerName
  , _wdQueue  :: !Broker.Queue
  , _wdBroker :: !WorkerBroker
  } deriving (Show, Eq)

data WorkerBroker =
  WorkerBrokerRedis WorkerRedis
  -- TODO Add WorkerBrokerPGMQ
  deriving (Show, Eq)

data WorkerRedis =
  WorkerRedis {
    _wrHost  :: !Text
  , _wrPort  :: !Int
  } deriving (Show, Eq)

workerSettingsCodec :: TomlCodec WorkerSettings
workerSettingsCodec = WorkerSettings
  <$> Toml.list workerDefinitionCodec "definitions" .= _wsDefinitions

workerDefinitionCodec :: TomlCodec WorkerDefinition
workerDefinitionCodec = WorkerDefinition
  <$> Toml.text "name" .= _wdName
  <*> Toml.string "queue" .= _wdQueue
  <*> Toml.table workerBrokerCodec "broker.redis" .= _wdBroker

workerBrokerCodec :: TomlCodec WorkerBroker
workerBrokerCodec =
  Toml.dimatch matchWorkerBrokerRedis WorkerBrokerRedis workerRedisCodec

matchWorkerBrokerRedis :: WorkerBroker -> Maybe WorkerRedis
matchWorkerBrokerRedis (WorkerBrokerRedis wr) = Just wr

workerRedisCodec :: TomlCodec WorkerRedis
workerRedisCodec = WorkerRedis
  <$> Toml.text "host" .= _wrHost
  <*> Toml.int "port" .= _wrPort


wdToRedisConnectInfo :: WorkerDefinition -> Maybe Redis.ConnectInfo
wdToRedisConnectInfo (WorkerDefinition { _wdBroker = WorkerBrokerRedis (WorkerRedis { .. }) }) =
  Just $ Redis.defaultConnectInfo { Redis.connectHost = T.unpack _wrHost
                                  , Redis.connectPort = Redis.PortNumber $ fromIntegral _wrPort }

findDefinitionByName :: WorkerSettings -> WorkerName -> Maybe WorkerDefinition
findDefinitionByName (WorkerSettings { _wsDefinitions }) workerName =
  head $ filter (\wd -> _wdName wd == workerName) _wsDefinitions

-- wdToRedisBrokerInitParams :: WorkerDefinition -> Maybe BRedis.RedisBrokerInitParams
-- wdToRedisBrokerInitParams wd = BRedis.RedisBrokerInitParams <$> (wdToRedisConnectInfo wd)

